/*
Copyright 2020 Appboxo pte. ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import 'dart:async';

import 'package:flutter/services.dart';

class AppboxoSdk {
  static const MethodChannel _channel = const MethodChannel('appboxo_sdk');
  static const EventChannel _customEventChannel =
      const EventChannel('appboxo_sdk_custom_events');
  static const EventChannel _lifecycleEventChannel =
      const EventChannel('appboxo_sdk_lifecycle');

  static void setConfig(String clientId) {
    _channel.invokeMethod("setConfig", {'client_id': clientId});
  }

  static void openMiniApp(
    String appId,
    String payload, {
    Map<String, String> additionalUserFields,
    Map<String, dynamic> data,
  }) {
    _channel.invokeMethod("openMiniApp", {
      'app_id': appId,
      'payload': payload,
      'additional_user_fields': additionalUserFields,
      'data': data
    });
  }

  static void hideAllMiniApps() {
    _channel.invokeMethod("hideAllMiniApps");
  }

  static Stream<CustomEvent> customEvents() {
    return _customEventChannel
        .receiveBroadcastStream()
        .map((dynamic event) => CustomEvent.fromJson(event));
  }

  static Future<void> Function() lifecycleHooksListener(
      {Function(String appId) onLaunch,
      Function(String appId) onResume,
      Function(String appId) onPause,
      Function(String appId) onClose,
      Function(String appId, String error) onError}) {
    StreamSubscription<LifecycleHook> subscription = _lifecycleEventChannel
        .receiveBroadcastStream()
        .map((event) => LifecycleHook.fromJson(event))
        .listen((hook) {
      switch (hook.lifecycle) {
        case MiniappLifecycleHooks.OnLaunch:
          if (onLaunch != null) {
            onLaunch(hook.appId);
          }
          break;
        case MiniappLifecycleHooks.OnResume:
          if (onResume != null) {
            onResume(hook.appId);
          }
          break;
        case MiniappLifecycleHooks.OnPause:
          if (onPause != null) {
            onPause(hook.appId);
          }
          break;
        case MiniappLifecycleHooks.OnClose:
          if (onClose != null) {
            onClose(hook.appId);
          }
          break;
        case MiniappLifecycleHooks.OnError:
          if (onError != null) {
            onError(hook.appId, hook.error);
          }
          break;
      }
    });

    return () => subscription.cancel();
  }

  static void sendEvent(CustomEvent customEvent) {
    _channel.invokeMethod("sendEvent", {'custom_event': customEvent.toJson()});
  }
}

class LifecycleHook {
  String appId;
  String lifecycle;
  String error;

  LifecycleHook({this.appId, this.lifecycle, this.error});

  LifecycleHook.fromJson(Map<dynamic, dynamic> json)
      : appId = json['appId'],
        lifecycle = json['lifecycle'],
        error = json['error'];

  Map<String, dynamic> toJson() => {
        'appId': appId,
        'lifecycle': lifecycle,
        'error': error,
      };
}

class CustomEvent {
  String appId;
  String requestId;
  String type;
  String errorType;
  Map payload;

  CustomEvent(
      this.appId, this.requestId, this.type, this.errorType, this.payload);

  CustomEvent.fromJson(Map<dynamic, dynamic> json)
      : appId = json['appId'],
        requestId = json['requestId'],
        type = json['type'],
        errorType = json['errorType'],
        payload = json['payload'];

  Map<String, dynamic> toJson() => {
        'appId': appId,
        'requestId': requestId,
        'type': type,
        'errorType': errorType,
        'payload': payload
      };
}

class MiniappLifecycleHooks {
  static const OnLaunch = 'onLaunch';
  static const OnResume = 'onResume';
  static const OnPause = 'onPause';
  static const OnClose = 'onClose';
  static const OnError = 'onError';
}
